;;; $Id: archive.lisp 403 2003-08-07 03:56:18Z miles $

(in-package :macho)

(defclass archive ()
  ((path :accessor archive-path :initarg :path)
   (name :accessor archive-name :initarg :name)
   (dirty-table :accessor archive-dirty-table)
   (last-pos :accessor archive-last-pos :initform 0)
   (messages :accessor archive-messages)
   (index-path :accessor archive-index-path)
   (mbox-path :accessor archive-mbox-path)
   (mbox :accessor archive-mbox)
   (year-table :accessor archive-year-table)))

(defmethod initialize-instance :after ((archive archive) &rest args)
  (declare (ignore args))
  (with-slots (path name dirty-table mbox mbox-path index-path) archive
    (setf name (pathname-name (archive-path archive))
          dirty-table (make-hash-table)
          mbox-path (join-path path "mbox")
          index-path (join-path path "index")
          mbox (open (archive-mbox-path archive) :direction :input))))

(defun archive-load (path)
  (let ((archive (make-instance 'archive :path path))
        (id-table (make-hash-table :test 'equal)))
    (with-slots (name messages year-table dirty-table) archive
      (setf year-table (make-hash-table))
      (index-update archive)
      (loop for msg across messages
            do (let* ((year (time-year (message-time msg)))
                      (month (time-month (message-time msg)))
                      (id (message-id msg))
                      (in-reply (message-in-reply msg)))
                 ;; put in year hash
                 (or (gethash year year-table)
                     (setf (gethash year year-table) (make-array 12 :initial-element nil)))
                 (push msg (svref (gethash year year-table) (1- month)))
                 ;; mark dirty table if dirty
                 (if (message-dirty msg)
                     (progn
                       (or (gethash year dirty-table)
                           (setf (gethash year dirty-table) (make-array 12 :initial-element nil)))
                     (setf (svref (gethash year dirty-table) (1- month)) t)))
                 ;; record references
                 (setf (gethash id id-table) msg)
                 (let ((parent (gethash in-reply id-table)))
                   (if parent
                       (progn
                         (push msg (message-children parent))
                         (setf (message-dirty parent) (message-dirty msg))
                         (setf (message-parent msg) parent))))))
      archive)))

(defun archive-process (archive)
  (write-archive archive (join-path *archive-dir* (archive-name archive))))

(defun process-archive (path)
  (let ((a (archive-load path)))
    (archive-process a)))
