\;;; -*- lisp -*-
;;; $Id: message.lisp 403 2003-08-07 03:56:18Z miles $

(in-package :macho)

(defvar *message-id-re* "<([^>]+)>")

(defun trim-email-addresses (text)
  (regex-replace-all "(.+)\\s*<?\\S+@[^ \\t\\r\\n>]+>?" text "\\1"))

(defun obscure-email-addresses (text)
  (regex-replace-all "<?(\\S+)@([^ \\t\\r\\n>]+)>?" text "&lt;\\1 at \\2&gt;"))

(defclass message ()
  ((archive :accessor message-archive)
   (author :accessor message-author)
   (body :initform nil)
   (children :accessor message-children :initform nil)
   (dirty :accessor message-dirty :initform nil)
   (end :accessor message-end :initarg :end)
   (headers :accessor message-headers :initarg :headers)
   (id :accessor message-id)
   (in-reply :accessor message-in-reply :initform nil)
   (parent :accessor message-parent :initform nil)
   (serial :accessor message-serial :initarg :serial)
   (start :accessor message-start :initarg :start)
   (subject :accessor message-subject)
   (text :accessor message-text :initarg :text)
   (time :accessor message-time)))

(defun chop-mime-message (message)
  "Chops off mime attachments. XXX: Hacky!"
  (with-slots (body headers) message
    (let* ((ctype (car (gethash :content-type headers)))
           match matches parts)
      (setf (values match matches) (scan-to-strings "boundary=\"([^\"]+)\"" ctype))
      (if match
          (progn
            (setf parts (split (s+ "--" (elt matches 0)) body))
            (setf body (second (split "\\n\\n" (second parts) :limit 2)))
            (setf body (regex-replace-all "=\\n" body ""))
            (setf body (regex-replace-all "=20" body ""))))))
  message)

(defun markup-body (body)
  (regex-replace-all "(http|ftp)://([-0-9A-Za-z#%&+./:;?_~\]*[-0-9A-Za-z#%&+/:;?_~\])" body "<a href=\"\\1://\\2\">http://\\2</a>"))

(defun message-body (message)
  (with-slots (body archive headers start end) message
    (unless body
      (let* ((mbox (archive-mbox archive))
             (size (- end start))
             (buffer (make-string size))
             full-message)
        (file-position mbox start)
        (read-sequence buffer mbox)
        (setf full-message (parse-message buffer))
        (setf body (slot-value full-message 'body))))
    body))

(defun parse-message (text)
  (let* ((parts (split "\\n\\n" text :limit 2))
         (header (car parts))
         (rawbody (cadr parts))
         (header-lines (split "\\n" (regex-replace-all "\\n\\s+" header " ")))
         (headers (make-hash-table :test #'equal))
         (msg (make-instance 'message :headers headers)))
    (dolist (header-line header-lines)
      (let* ((hdrparts (split ":\\s+" header-line :limit 2))
             (key (intern (string-upcase (car hdrparts)) 'keyword))
             (val (cadr hdrparts)))
        (push val (gethash key headers))))
    (with-slots (author body headers id in-reply subject time) msg
      (setf time (time-struct
                  (or (net.telent.date:parse-time
                       (cadr (split ";\\s*" (first (last (gethash :received headers))))))
                      (get-universal-time)))
            author (trim-email-addresses (first (gethash :from headers)))
            body (markup-body rawbody)
            id (regex-replace-all *message-id-re* (first (gethash :message-id headers)) "\\1")
            in-reply (regex-replace-all *message-id-re*
                                        (if (gethash :in-reply-to headers)
                                            (first (split "\\s+" (first (gethash :in-reply-to headers))))
                                            (first (last (split "\\s+" (first (gethash :references headers))))))
                                        "\\1")
            subject (first (gethash :subject headers))))
    (chop-mime-message msg)))
